#include "main.h"

/*
 *  Variables
 */
volatile uint8_t new_capture;



/*
 * System Init
 */
void system_init(void)
{

  LL_RCC_HSI_Enable();

  /* Change this value to adjust clock frequency, larger is faster */
  LL_RCC_HSI_SetCalibFreq(LL_RCC_HSICALIBRATION_24MHz);
  while (LL_RCC_HSI_IsReady() != 1);

  // Enable Clocks
  LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOA | LL_IOP_GRP1_PERIPH_GPIOB | LL_IOP_GRP1_PERIPH_GPIOF);
  LL_APB1_GRP2_EnableClock(LL_APB1_GRP2_PERIPH_TIM1);

  // GPIO init
  WritePin(RED,1);
  WritePin(BLUE,1);
  LL_GPIO_Init(CCP_Port,  &(LL_GPIO_InitTypeDef){ CCP_Pin,  LL_GPIO_MODE_ALTERNATE, LL_GPIO_SPEED_FREQ_VERY_HIGH, LL_GPIO_OUTPUT_PUSHPULL, LL_GPIO_PULL_NO, CCP_AF });
  LL_GPIO_Init(RED_Port,  &(LL_GPIO_InitTypeDef){ RED_Pin,  LL_GPIO_MODE_OUTPUT,    LL_GPIO_SPEED_FREQ_VERY_HIGH, LL_GPIO_OUTPUT_PUSHPULL });
  LL_GPIO_Init(BLUE_Port, &(LL_GPIO_InitTypeDef){ BLUE_Pin, LL_GPIO_MODE_OUTPUT,    LL_GPIO_SPEED_FREQ_VERY_HIGH, LL_GPIO_OUTPUT_PUSHPULL });

  // TIMER init
  LL_TIM_Init(CCP_TIM, &(LL_TIM_InitTypeDef){ 23, LL_TIM_COUNTERDIRECTION_UP, 65535, LL_TIM_CLOCKDIVISION_DIV1, 0});                              // 1MHz clock, 65ms overflow

  // TIMER IC init
  LL_TIM_IC_Init(CCP_TIM, CCP_CH, &(LL_TIM_IC_InitTypeDef){ LL_TIM_IC_POLARITY_FALLING, LL_TIM_ACTIVEINPUT_DIRECTTI, LL_TIM_ICPSC_DIV1, 4 });      // Falling edge
  LL_TIM_EnableCounter(CCP_TIM);

  // NVIC init
  NVIC_SetPriority(TIM1_CC_IRQn, 2);
  NVIC_EnableIRQ(TIM1_CC_IRQn);

  // Systick
  SysTick_Config(24000);                                                                  // 24MHz cpu clock, 1ms period
}

void capture_start(void) {
  LL_TIM_ClearFlag_CC4(CCP_TIM);
  LL_TIM_EnableIT_CC4(CCP_TIM);
}


/*
 * RDP lock
 */
static void lock_RDP(void)
{
  FLASH_OBProgramInitTypeDef pOBInit;
  LL_FLASH_OBGetConfig(&pOBInit);
  if(pOBInit.RDPLevel != OB_RDP_LEVEL_1)
  {
    pOBInit.OptionType = OPTIONBYTE_RDP;
    pOBInit.RDPLevel = OB_RDP_LEVEL_1;
    LL_FLASH_Unlock();
    LL_FLASH_OB_Unlock();
    LL_FLASH_OBProgram(&pOBInit);
    LL_FLASH_OB_Launch();
    NVIC_SystemReset();
  }
}


/*
 * Main
 */
int main(void)
{
  system_init();
  lock_RDP();           // <<--- Enabling RDP causes random system crash when toggling CCP input
  capture_start();
  while(1)
  {
  }
}



/*
 * Systick interrupt
 */
void systick_isr(void)
{
  static uint16_t b,r;

  if(new_capture)
  {                               // Turn on BLUE LED for 10ms when a command is received.
    new_capture=0;
    WritePin(RED,0);
    r=10;
    capture_start();              // Start a new capture
  }
  else if(r && !--r)
  {
    WritePin(RED,1);              // Led expired
  }

  if(!b--)
  {                               // 10ms pulse on RED LED every second
    bool led = ReadPin(BLUE);
    WritePin(BLUE,!led);
    b = led ? 10 : 990;           // 10ms on, 990ms off
  }
}

/*
 * Capture interrupt
 */
void capture_isr(void)
{
  LL_TIM_SetCounter(CCP_TIM, 0);                                                      // Reset timer counter
  if(LL_TIM_IsActiveFlag_CC4(CCP_TIM))                                                // Capture flag active
  {
    LL_TIM_IC_GetCaptureCH4(CCP_TIM);                                                 // Capture CCP value
    LL_TIM_DisableIT_CC4(CCP_TIM);                                                    // Disable CC interrupt
    LL_TIM_ClearFlag_CC4(CCP_TIM);                                                    // Clear flag
    new_capture=1;
  }
}
